# frozen_string_literal: true

require "spec_helper"

RSpec.describe "Bootstraps new project with default configuration" do
  before(:all) do
    drop_dummy_database
    remove_project_directory
    run_tractor_beam
  end

  it "uses custom Gemfile" do
    gemfile_file = IO.read("#{project_path}/Gemfile")
    expect(gemfile_file).to match(
      /^ruby "#{TractorBeam::RUBY_VERSION}"$/,
    )
    expect(gemfile_file).to match(
      /^gem "rails", "#{TractorBeam::RAILS_VERSION}"$/,
    )
  end

  it "Rakefile includes assets customization" do
    rakefile = IO.read("#{project_path}/Rakefile")
    expect(rakefile).to include(
      'Rake::Task.define_task("assets:precompile" => ["yarn:install", "webpacker:compile"])',
    )
  end

  it "rspec files are included in the project" do
    expect(File).to exist("#{project_path}/spec/spec_helper.rb")
    expect(File).to exist("#{project_path}/spec/rails_helper.rb")
  end

  it "includes simplecov configuration" do
    rails_helper = IO.read("#{project_path}/spec/rails_helper.rb")
    expect(rails_helper).to include("require 'simplecov'")
  end

  it "includes FactoryBot configuration" do
    rails_helper = IO.read("#{project_path}/spec/rails_helper.rb")
    expect(rails_helper).to include("config.include FactoryBot::Syntax::Methods")
  end

  it "includes ShouldaMatchers configuration" do
    rails_helper = IO.read("#{project_path}/spec/rails_helper.rb")
    expect(rails_helper).to include("Shoulda::Matchers.configure")
  end

  it "creates .ruby-version from TractorBeam .ruby-version" do
    ruby_version_file = IO.read("#{project_path}/.ruby-version")

    expect(ruby_version_file).to eq "#{RUBY_VERSION}\n"
  end

  # it "copies dotfiles" do
  #   %w[.ctags .env].each do |dotfile|
  #     expect(File).to exist("#{project_path}/#{dotfile}")
  #   end
  # end

  it "doesn't generate test directory" do
    expect(File).not_to exist("#{project_path}/test")
  end

  # it "loads secret_key_base from env" do
  #   secrets_file = IO.read("#{project_path}/config/secrets.yml")
  #
  #   expect(secrets_file).
  #     to match(/secret_key_base: <%= ENV\["SECRET_KEY_BASE"\] %>/)
  # end

  it "adds bin/setup file" do
    expect(File).to exist("#{project_path}/bin/setup")
  end

  it "makes bin/setup executable" do
    bin_setup_path = "#{project_path}/bin/setup"

    expect(File.stat(bin_setup_path)).to be_executable
  end

  # it "adds support file for action mailer" do
  #   expect(File).to exist("#{project_path}/spec/support/action_mailer.rb")
  # end

  # it "configures capybara-webkit" do
  #   expect(File).to exist("#{project_path}/spec/support/capybara_webkit.rb")
  # end

  # it "adds support file for i18n" do
  #   expect(File).to exist("#{project_path}/spec/support/i18n.rb")
  # end

  # it "initializes ActiveJob to avoid memory bloat" do
  #   expect(File).
  #     to exist("#{project_path}/config/initializers/active_job.rb")
  # end

  # fit "raises on unpermitted parameters in all environments" do
  #   result = IO.read("#{project_path}/config/application.rb")
  #
  #   expect(result).to match(
  #     /^ +config.action_controller.action_on_unpermitted_parameters = :raise$/,
  #   )
  # end

  # it "configures production environment to enforce SSL" do
  #   expect(production_config).to match(
  #     /^ +config.force_ssl = true/,
  #   )
  # end

  # it "configs simple_form" do
  #   expect(File).to exist("#{project_path}/config/initializers/simple_form.rb")
  # end

  # it "configs :test email delivery method for development" do
  #   expect(development_config).
  #     to match(/^ +config.action_mailer.delivery_method = :file$/)
  # end

  # it "sets action mailer default host and asset host" do
  #   config_key = 'config\.action_mailer\.asset_host'
  #   config_value =
  #     %q{ENV\.fetch\("ASSET_HOST", ENV\.fetch\("APPLICATION_HOST"\)\)}
  #   expect(production_config).to match(/#{config_key} = #{config_value}/)
  # end

  # it "uses APPLICATION_HOST, not HOST in the production config" do
  #   expect(production_config).to match(/"APPLICATION_HOST"/)
  #   expect(production_config).not_to match(/"HOST"/)
  # end

  # it "configures email interceptor in smtp config" do
  #   smtp_file = IO.read("#{project_path}/config/smtp.rb")
  #   expect(smtp_file).
  #     to match(/RecipientInterceptor.new\(ENV\["EMAIL_RECIPIENTS"\]\)/)
  # end

  it "adds spring to binstubs" do
    expect(File).to exist("#{project_path}/bin/spring")

    bin_stubs = %w(rake rails rspec)
    bin_stubs.each do |bin_stub|
      expect(IO.read("#{project_path}/bin/#{bin_stub}")).to match(/spring/)
    end
  end

  # it "removes comments and extra newlines from config files" do
  #   config_files = [
  #     IO.read("#{project_path}/config/application.rb"),
  #     IO.read("#{project_path}/config/environment.rb"),
  #     development_config,
  #     test_config,
  #     production_config,
  #   ]
  #
  #   config_files.each do |file|
  #     expect(file).not_to match(/.*#.*/)
  #     expect(file).not_to match(/^$\n/)
  #   end
  # end

  # it "copies factories.rb" do
  #   expect(File).to exist("#{project_path}/spec/factories.rb")
  # end

  def app_name
    TractorBeamTestHelpers::APP_NAME
  end

  # it "configure modern frontend tools" do
  #   flashes_path = %w(app assets stylesheets refills _flashes.scss)
  #   expect(read_project_file(flashes_path)).to match(/\$flashes/m)
  #
  #   app_css = read_project_file(%w(app assets stylesheets application.scss))
  #   expect(app_css).to match(
  #     /normalize\.css\/normalize\.css.*bourbon.*neat.*base.*refills/m,
  #   )
  # end

  it "doesn't use turbolinks" do
    js_file = read_project_file(%w(frontend init index.js))
    expect(js_file).not_to match(/Turbolinks/)
  end

  it "use rails-ujs" do
    js_file = read_project_file(%w(frontend init index.js))
    expect(js_file).to match(/Rails/)
  end

  it "setup actioncable" do
    js_file = read_project_file(%w(frontend client cable.js))
    expect(js_file).to match(/actioncable/)
  end

  def development_config
    @_development_config ||=
      read_project_file %w(config environments development.rb)
  end

  def test_config
    @_test_config ||= read_project_file %w(config environments test.rb)
  end

  def production_config
    @_production_config ||=
      read_project_file %w(config environments production.rb)
  end

  def read_project_file(path)
    IO.read(File.join(project_path, *path))
  end
end
